import { LogMessage, LogLevel } from './log-message';
import LogAppender from './log-appender';

export class Logger {

  public appenders: LogAppender[] = [];

  public constructor(
    public readonly name: string,
    private parentLogger?: Logger,
    public threshold?: LogLevel) { }

  public createChildLogger(name: string, threshold?: LogLevel): Logger {
    return new Logger(name, this, threshold);
  }

  public doLog(level: LogLevel, message: (Error | string), stacktrace?: string): void {

    if (level < this.getEffectiveThreshold()) { return; }

    const logMsg: LogMessage = {
      scope: this.name,
      level,
      message: '',
      stacktrace: '',
      timestamp: new Date() };

    if (typeof message === 'string') {
      logMsg.message = message;
      logMsg.stacktrace = stacktrace == null ? '' : stacktrace;
    } else {
      logMsg.error = message;
      logMsg.message = `${message.name}: ${message.message}`;
      logMsg.stacktrace = message.stack == null ? '' : message.stack;
    }

    this.sendToAppenders(logMsg);
  }

  public trace(message: (Error | string), stacktrace?: string): void {
    this.doLog(LogLevel.ALL, message, stacktrace);
  }

  public debug(message: (Error | string), stacktrace?: string): void {
    this.doLog(LogLevel.DEBUG, message, stacktrace);
  }

  public log(message: string, stacktrace?: string): void {
    this.doLog(LogLevel.LOG, message, stacktrace);
  }

  public info(message: string, stacktrace?: string): void {
    this.doLog(LogLevel.INFO, message, stacktrace);
  }

  public warn(message: string, stacktrace?: string): void {
    this.doLog(LogLevel.WARN, message, stacktrace);
  }

  public error(message: string, stacktrace?: string): void {
    this.doLog(LogLevel.ERROR, message, stacktrace);
  }

  public fatal(message: string, stacktrace?: string): void {
    this.doLog(LogLevel.FATAL, message, stacktrace);
  }

  protected sendToAppenders(logMsg: LogMessage) {
    this.appenders.forEach((app) => {
      app.appendMessage(logMsg);
    });

    if (this.parentLogger) { this.parentLogger.sendToAppenders(logMsg); }
  }

  protected getEffectiveThreshold(): LogLevel {
    if (this.threshold) { return this.threshold; }
    if (this.parentLogger) { return this.parentLogger.getEffectiveThreshold(); }

    // should never happen (root logger should always have a threshold
    return LogLevel.ALL;
  }

}

export default Logger;
